class Pettanr.Views.Scroll.PlayModule
class Pettanr.Views.Scroll.PlayModule.Header extends Pettanr.Views.Show.Header
  
  initialize: (options) ->
    super(options)
  
class Pettanr.Views.Scroll.PlayModule.Owner extends Pettanr.Views.Show.Owner
  
  initialize: (options) ->
    super(options)
  
class Pettanr.Views.Scroll.PlayModule.FooterSwitch extends Backbone.View
  tagName: 'div'
  className: 'player-item-switch'
  events: {
    'click': 'click'
  }
  
  initialize: (options) ->
    @scroll_panel = options.scroll_panel
    @panel = options.panel
    @visible = false
  
  render: () ->
    this.$el.html('')
    attr = {
      style: Pettanr.to_style({height: Pettanr.to_s(@panel.get('height')) + 'px'})
    }
    this.$el.attr(attr)
    face = if @visible
      '<'
    else
      '>'
    caption = new Tag.Span({content: face})
    this.$el.append(@scroll_panel.get('t') + 1)
    this.$el.append(caption.render().el)
    this
  
  click: () ->
    @visible = !@visible
    @render()
    @trigger('click', @visible)
  
class Pettanr.Views.Scroll.PlayModule.Footer extends Backbone.View
  tagName: 'div'
  className: 'player-item-footer'
  
  initialize: (options) ->
    @scroll = options.scroll
    @scroll_panel = options.scroll_panel
    @panel = options.panel
  
  render: () ->
    this.$el.html('')
    @hide()
    retriever = @panel.author()
    @listenTo(retriever, 'retrieve', @retrieve_author)
    retriever.retrieve()
    this
  
  retrieve_author: (@author) -> 
    @panel_icon = new Pettanr.Views.Panel.Icon({item: @panel, half: true})
    @author_icon = new Pettanr.Views.Author.Icon({item: @author, half: true})
    @new_scroll_panel = new Pettanr.ScrollPanel()
    @insert_button = new Pettanr.Views.Show.LinkedNewButton({item: @new_scroll_panel, dic_name: 'scroll_panels.player.before_insert'})
    @move_button = new Pettanr.Views.Show.LinkedEditButton({item: @scroll_panel, dic_name: 'scroll_panels.player.move'})
    @destroy_button = new Pettanr.Views.Show.LinkedDestroyButton({item: @scroll_panel, dic_name: 'scroll_panels.player.destroy'})
    @listenTo(@panel_icon, 'click', @click_panel)
    @listenTo(@author_icon, 'click', @click_author)
    @listenTo(@insert_button, 'click', @click_insert)
    @listenTo(@move_button, 'click', @click_move)
    @listenTo(@destroy_button, 'click', @click_destroy)
    l1 = new Tag.H3({
      content: Pettanr.AppHelper.t_m('Panel')
    })
    l2 = new Tag.H3({
      content: Pettanr.AppHelper.t_m('ScrollPanel')
    })
    this.$el.append(l1.render().el)
    this.$el.append(@panel_icon.render().el)
    this.$el.append(@author_icon.render().el)
    this.$el.append(
      Pettanr.AppHelper.distance_of_time_in_words_to_now(@panel.get('updated_at'))
    )
    this.$el.append(l2.render().el)
    if @scroll.is_own()
      this.$el.append(@insert_button.render().el)
      rb = new Tag.RowBreak()
      this.$el.append(rb.render().el)
      this.$el.append(@move_button.render().el)
      rb = new Tag.RowBreak()
      this.$el.append(rb.render().el)
      this.$el.append(@destroy_button.render().el)
  
  hide: () ->
    attr = {
      style: Pettanr.to_style({display: 'none'})
    }
    this.$el.attr(attr)
  
  show: () ->
    attr = {
      style: Pettanr.to_style({display: 'inline'})
    }
    this.$el.attr(attr)
  
  click_panel: () ->
    @trigger('click:panel')
  
  click_author: () ->
    @trigger('click:author')
  
  click_insert: () ->
    @trigger('click:insert')
  
  click_move: () ->
    @trigger('click:move')
  
  click_destroy: () ->
    @trigger('click:destroy')
  
class Pettanr.Views.Scroll.PlayModule.Credits extends Backbone.View
  tagName: 'div'
  className: 'credits'
  
  initialize: (options) ->
    @parent = options.parent
    @licensed_pictures = {}
  
  render: () ->
    this.$el.html('')
    this
  
  push: (retrievers) ->
    _.each retrievers, (retriever) =>
      @listenTo(retriever, 'retrieve', @retrieve_picture)
      retriever.retrieve()
  
  retrieve_picture: (picture) ->
    pid = picture.get('id')
    return if @licensed_pictures[pid]
    @licensed_pictures[pid] = picture
    credit = picture.credit_view()
    @listenTo(credit, 'click:icon', @click_credit_icon)
    this.$el.append(credit.render().el)
  
  click_credit_icon: (item) ->
    @trigger('click:credit:icon', item)
  
class Pettanr.Views.Scroll.PlayModule.PanelBody extends Backbone.View
  tagName: 'div'
  className: 'player-item-panel'
  
  initialize: (options) ->
    @panel = options.panel
  
  render: () ->
    this.$el.html('')
    body = new Pettanr.Views.Panel.Body({
      panel: @panel,
      spot: null
    })
    this.$el.append(body.render().el)
    this
  
class Pettanr.Views.Scroll.PlayModule.Panel extends Backbone.View
  tagName: 'li'
  className: 'player-item'
  
  initialize: (options) ->
    @scroll = options.scroll
    @scroll_panel = options.scroll_panel
  
  clear: () ->
    this.$el.html('')
    retriever = @scroll_panel.panel()
    @listenTo(retriever, 'retrieve', @retrieve_panel)
    retriever.retrieve()
    this
  
  retrieve_panel: (@panel) ->
    @render()
  
  render: () ->
    this.$el.html('')
    @panel = @panel.with_elements() # retake panel for 'with_elements' mode
    @panel.fetch({cache: true}).done =>
      @panel.attributes = @panel.replaced_attributes()
      body = new Pettanr.Views.Scroll.PlayModule.PanelBody({
        panel: @panel
      })
      btn = new Pettanr.Views.Scroll.PlayModule.FooterSwitch({
        scroll_panel: @scroll_panel,
        panel: @panel
      })
      @listenTo(btn, 'click', @click_footer_switch)
      @footer = new Pettanr.Views.Scroll.PlayModule.Footer({
        scroll: @scroll,
        scroll_panel: @scroll_panel,
        panel: @panel,
      })
      @listenTo(@footer, 'click:panel', @click_show)
      @listenTo(@footer, 'click:author', @click_author)
      @listenTo(@footer, 'click:insert', @click_insert)
      @listenTo(@footer, 'click:move', @click_move)
      @listenTo(@footer, 'click:edit', @click_edit)
      @listenTo(@footer, 'click:destroy', @click_destroy)
      this.$el.append(body.render().el)
      this.$el.append(btn.render().el)
      this.$el.append(@footer.render().el)
      @trigger('ready', @panel)
    this
  
  hide: () ->
    @set_style({display: 'none'})
  
  show: () ->
    @set_style({display: 'inline'})
  
  set_style: (style) ->
    attr = {style: Pettanr.to_style(style)}
    this.$el.attr(attr)
  
  click_footer_switch: (visible) ->
    if visible
      @footer.show()
    else
      @footer.hide()
  
  click_show: () ->
    @trigger('http_get', @panel.show_url())
  
  click_author: () ->
    @trigger('http_get', @panel.author().show_url())
  
  click_insert: () ->
    @trigger('click:insert', this)
  
  click_move: () ->
    @trigger('click:move', this)
  
  click_destroy: () ->
    @trigger('click:destroy', this)
  
class Pettanr.Views.Scroll.PlayModule.Append extends Tag.Div
  
class Pettanr.Views.Scroll.PlayModule.Panels extends Backbone.View
  tagName: 'ul'
  className: 'player'
  
  initialize: (options) ->
    @parent = options.parent
    @binder = options.binder
    @items = options.items
    @credits = options.credits
    @views = []
  
  render: () ->
    this.$el.html('')
    _.each @items, (scroll_panel) =>
      return if not scroll_panel.has_panel()
      panel_view = @create_panel_view(scroll_panel)
      this.$el.append(panel_view.clear().el)
    rb = new Tag.RowBreak()
    this.$el.append(rb.render().el)
    link = new Tag.A({
      attr: {href: '/scroll_panels/new'},
      content: I18n.t('scroll_panels.player.append')
    })
    @listenTo(link, 'click', @click_append)
    @appender = new Pettanr.Views.Scroll.PlayModule.Append({
      content: link.render().el
    })
    this.$el.append(@appender.render().el)
    this
  
  create_panel_view: (scroll_panel) ->
    panel_view = new Pettanr.Views.Scroll.PlayModule.Panel({
      scroll: @binder, scroll_panel: scroll_panel,
    })
    @listenTo(panel_view, 'ready', @ready)
    @listenTo(panel_view, 'click:insert', @click_insert)
    @listenTo(panel_view, 'click:move', @click_move)
    @listenTo(panel_view, 'click:destroy', @click_destroy)
    @views.push(panel_view)
    panel_view
  
  ready: (panel) ->
    this.credits.push(panel.licensed_pictures())
  
  click_insert: (panel_view) ->
    @trigger('click:insert')
    @dialog = new Editor.Player.PanelInsertDialog({
      parent: this, binder: @binder, target_model: Pettanr.ScrollPanel,
      insert_point: panel_view
    })
    @listenTo(@dialog, 'pick', @pick)
    @listenTo(@dialog, 'dialog:open', @open_dialog)
    @listenTo(@dialog, 'dialog:close', @close_dialog)
    this.$el.append(@dialog.render().el)
    @dialog.start()
  
  click_append: () ->
    @click_insert(@appender)
  
  pick: (new_item, insert_point) ->
    panel_view = @create_panel_view(new_item)
    ul = this.$el[0]
    li = insert_point.$el[0]
    ul.insertBefore(panel_view.clear().el, li)
    @dialog.remove()
  
  open_dialog: (dialog) ->
    @trigger('dialog:open', dialog)
  
  close_dialog: (dialog) ->
    @trigger('dialog:close', dialog)
  
  click_move: (panel_view) ->
    @trigger('click:move')
    @dialog = new Editor.Player.PanelMoveDialog({
      parent: this
    })
    this.$el.append(@dialog.render().el)
    _this = this
    @dialog.$el.dialog({
      autoOpen: false,
      width: 500, 
      height: 500,
      close: (ui, event) ->
        _this.remove()
    })
    @listenTo(@dialog, 'success', @success)
    @dialog.start(panel_view.panel)
    panel_view.hide()
  
  show_insert_point: () ->
    _.each @views, (panel_view) =>
      panel_view.show_insert_point
  
  click_destroy: (panel_view) ->
    success = (model, response) =>
      panel_view.remove()
      @trigger('click:destroy')
    Pettanr.Proxy.destroy(panel_view.scroll_panel, {success: success})
  
  remove_panel_view: (panel_view) ->
    @views = _.without(@views, panel_view)
    panel_view.remove()
  
class Pettanr.Views.Scroll.PlayModule.Body extends Backbone.View
  tagName: 'div'
  
  initialize: (options) ->
    @parent = options.parent
    @binder = options.binder
    @list = options.list
    @items = @list.items()
    if @items.length < 1
      @render_empty()
    else
      @render()
  
  render_empty: () ->
    this.$el.html('')
    this.$el.append((new Tag.H2({content: I18n.t('scrolls.play.empty')})).render().el)
    this
  
  render: () ->
    this.$el.html('')
    # paginate(@pager)
    credits = new Pettanr.Views.Scroll.PlayModule.Credits({parent: this})
    panels = new Pettanr.Views.Scroll.PlayModule.Panels({
      parent: this,
      binder: @binder,
      items: @items,
      credits: credits,
    })
    @listenTo(credits, 'click:credit:icon', @click_credit_icon)
    this.$el.append(panels.render().el)
    this.$el.append(credits.render().el)
    rb = new Tag.RowBreak()
    this.$el.append(rb.render().el)
    # paginate(@pager)
    this
  
  click_credit_icon: (item) ->
    @trigger('http_get', item.show_url())
  
class Pettanr.Views.Scroll.Play extends Backbone.View
  tagName: 'div'
  className: 'show'
  
  initialize: (options) ->
    @item = options.item
    @list = options.list
    icon_url = Pettanr.url(@item.table_name(), 'show', {id: @item.get('id')})
    caption_url = Pettanr.url(@item.table_name(), 'play', {id: @item.get('id')})
    prof_url = Pettanr.url(@item.table_name(), 'show', {id: @item.get('id'), format: 'prof'})
    @header = new Pettanr.Views.Scroll.PlayModule.Header({
      item: @item, 
      caption: @item.get('title'), 
      icon_url: icon_url,
      caption_url: caption_url,
      prof_url: prof_url
    })
    @author = new Pettanr.Views.Show.HeaderAuthor({item: @item})
    @body = new Pettanr.Views.Scroll.PlayModule.Body({
      parent: this,
      binder: @item,
      list: @list, 
    })
    # @owner = new Pettanr.Views.Scroll.PlayModule.Owner({item: @item})
    @listenTo(@header, 'click:pick', @click_pick)
    @listenTo(@header, 'click:icon', @click_show)
    @listenTo(@header, 'click:caption', @click_play)
    @listenTo(@header, 'click:prof', @click_prof)
    @listenTo(@author, 'click:author', @click_author)
    @listenTo(@body, 'http_get', @http_get)
  
  render: () ->
    this.$el.html('')
    this.$el.append(@header.render().el)
    this.$el.append(@author.render().el)
    this.$el.append(@body.render().el)
    # this.$el.append(@owner.render().el) if @item.is_own()
    this
  
  click_show: () ->
    @trigger('http_get', @item.show_url())
  
  click_play: () ->
    @trigger('http_get', @item.play_url())
  
  click_prof: () ->
    @trigger('http_get', @item.prof_url())
  
  click_author: () ->
    @trigger('http_get', @author.author.show_url())
  
  click_pick: () ->
    @trigger('pick', @item)
  
  http_get: (url) ->
    @trigger('http_get', url)
  
