class UserSessionsController < Devise::SessionsController
  prepend_before_action :require_no_authentication, :only => [:create ]
  respond_to :json

  def create
    if request.xhr?
      resource = warden.authenticate!(scope: resource_name, recall: "#{controller_path}#failure")
      sign_in(resource_name, resource)
      return render :json => current_user.to_json({:include => {:author => {}, :artist => {}} })
    else
      super
    end
  end
  
  def destroy
    if request.xhr?
      Devise.sign_out_all_scopes ? sign_out : sign_out(resource_name)
      return render :json => {:success => true}
    else
      super
    end
  end
  
  def xhr_failure
    return render :json => {:success => false, :errors => ["Login failed."]}
  end
  
  def require_no_authentication
    if request.xhr?
      true
    else
      super
    end
  end
  
end
