#サポートツール
require 'rubygems'
require 'active_support/all'
require 'rest_client'
require 'yaml'
require 'json'
require 'base64'

if ARGV.size < 1
  puts "uploader.rb host dirname attr.json lsname auth"
  puts "upload on child dirs"
  exit
end

host = ARGV[0]
dirname = ARGV[1]
attr_fn = ARGV[2]
lsname = ARGV[3]
auth = ARGV[4]

class Uploader
  def initialize host, dirname, attr_fn, lsname, auth
    @host = host
    @dirname = dirname
    @attr_fn = attr_fn
    @lsname = lsname
    @auth = auth
    @licenses = self.licenses
    @license_groups = self.license_groups
  end
  
  def licenses
    res = {}
    u = @host + 'licenses.json'
    r = RestClient.get(u)
    ary = JSON.parse r
    ary['list'].each do |l|
      res[l['name']] = l
    end
    res
  end

  def license_groups
    res = {}
    u = @host + 'license_groups.json'
    r = RestClient.get(u)
    ary = JSON.parse r
    ary['list'].each do |l|
      res[l['id'].to_i] = l
    end
    res
  end
  
  def each_subdir
    Dir.glob File.expand_path(@dirname) + '/*' do |filename|
      if File.directory?(filename)
        yield filename
      end
    end
  end
  
  def each_op
    Dir.glob File.expand_path(@dirname) + '/*' do |fn|
      ext = File.extname(fn).downcase
      case ext
      when '.png', '.gif', '.jpeg'
        yield fn
      end
    end
  end
  
  def upload fn
    oid = nil
    b = Base64.encode64(File.open(fn, 'rb').read)
    u = @host + 'original_pictures'
    begin
      r = RestClient.post(u, 
        {:original_picture => {:file  => b}, :auth_token => @auth}.to_json, 
        :content_type => :json, :accept => :json
      )
      o = JSON.parse r
      oid = o['id']
    rescue
      puts ' failed: ' + r.to_s
    end
    oid
  end
  
  def original_picture_license oid, lgid, lsid, en, attr
    rp = nil
    u = @host + '/original_picture_licenses'
    param = {
      :original_picture_id => oid, 
      :license_group_id => lgid, 
      :license_id => lsid, 
      :license_group_module_name => en, 
      :license_settings => attr, 
      :auth_token => @auth 
    }
    begin
      r = RestClient.post(u, 
        param.to_json,
        :content_type => :json, :accept => :json
      )
      rp = JSON.parse r
    rescue
      puts ' failed: ' + u.to_s
      puts param
      puts r.to_s
    end
    rp
  end
  
  #confirm
  def resource_picture oid, lgid, lsid, en, formname, attr
    rp = nil
    u = @host + en.tableize + '/attributes'
    attr['license_id'] = lsid
    param = {
      :original_picture_id => oid, 
      :original_picture_license_group => {
        :original_picture_id => oid, :license_group_id => lgid
      }, 
      :auth_token => @auth 
    }
    param[:attributes] = attr
    begin
      r = RestClient.post(u, 
        param.to_json,
        :content_type => :json, :accept => :json
      )
      rp = JSON.parse r
    rescue
      puts ' failed: ' + u.to_s
      puts param
      puts r.to_s
    end
    rp
  end
  
  def publish oid, lgid, rp
    res = nil
    u = @host + 'resource_pictures'
    attr = {
      :original_picture_id => oid, 
      :original_picture_license_group => {
        :original_picture_id => oid, :license_group_id => lgid
      }, 
      :resource_picture => rp, 
      :auth_token => @auth 
    }
    begin
      r = RestClient.post(u, 
        attr.to_json,
        :content_type => :json, :accept => :json
      )
      res = JSON.parse r
    rescue
      puts ' failed: ' + r.to_s
    end
    res
  end
  
  def publish_all
    attr = JSON.parse(File.open(@attr_fn).read)
    each_op do |imgfile|
      puts 'uploading: ' + imgfile
      oid = self.upload imgfile
      next unless oid
      puts 'success: ' + oid.to_s
      ls = @licenses[@lsname]
      unless ls
        puts 'not detect licenses: ' + @lsname.to_s
        next
      end
      lsid = ls['id']
      lgid = ls['license_group_id'].to_i
      lg = @license_groups[lgid]
      en = lg['module_name']
      confirm = self.original_picture_license oid, lgid, lsid, en, attr
      puts 'publishing: ' + @lsname.to_s
      r = self.publish oid, lgid, confirm
      puts 'success: ' + r['id'].to_s
    end
  end
  
end

u = Uploader.new host, dirname, attr_fn, lsname, auth

u.publish_all

